<?php

add_action( 'suptic_control_create_ticket', 'suptic_ticket_created_notification' );

function suptic_ticket_created_notification( &$ticket ) {
	if ( ! $ticket )
		return;

	$subject = sprintf( __( '[%s] Support Tickets - Ticket Created (TKT:%d)', 'suptic' ),
		get_option( 'blogname' ), $ticket->id );

	$body = __( "New support ticket created:", 'suptic' ) . "\n\n";
	$body .= $ticket->url( true );
	$body .= "\n\n";
	$body .= sprintf( __( 'Subject: %s', 'suptic' ), $ticket->subject ) . "\n";
	$body .= sprintf( __( 'Author: %1$s %2$s', 'suptic' ),
		$ticket->author_first_name, $ticket->author_last_name ) . "\n\n";
	$body .= sprintf( __( 'Ticket tracking: %s', 'suptic' ), $ticket->access_key ) . "\n";

	if ( $message = $ticket->first_message() ) {
		$body .= __( 'Message:', 'suptic' ) . "\n";
		$body .= $message->message_body . "\n\n";
	}

	if ( is_email( $ticket->author_email ) )
		@wp_mail( $ticket->author_email, $subject, $body );

	if ( ( $admin_email = get_option('admin_email') ) && $admin_email != $ticket->author_email )
		@wp_mail( $admin_email, $subject, $body );
}

add_action( 'suptic_control_add_message', 'suptic_message_replied_notification', 10, 2 );

function suptic_message_replied_notification( &$ticket, &$message ) {
	if ( ! $ticket || ! $message )
		return;

	if ( ! $message->has_status( 'publish' ) )
		return;

	$from_admin = $message->is_admin_reply();

	$subject = sprintf( __( '[%s] Support Tickets - You Got Reply (TKT:%d)', 'suptic' ),
		get_option( 'blogname' ), $ticket->id );

	$body = __( "You got new reply message:", 'suptic' ) . "\n\n";
	$body .= $ticket->url( true );
	$body .= "\n\n";
	$body .= sprintf( __( 'Subject: %s', 'suptic' ), $ticket->subject ) . "\n";

	if ( ! $from_admin ) {
		$body .= sprintf( __( 'Author: %1$s %2$s', 'suptic' ),
			$ticket->author_first_name, $ticket->author_last_name ) . "\n\n";
	}
	$body .= sprintf( __( 'Ticket tracking: %s', 'suptic' ), $ticket->access_key ) . "\n";

	$body .= __( 'Message:', 'suptic' ) . "\n";
	$body .= $message->message_body . "\n\n";

	if ( $from_admin && is_email( $ticket->author_email ) )
		@wp_mail( $ticket->author_email, $subject, $body );

	if ( ! $from_admin && ( $admin_email = get_option('admin_email') ) )
		@wp_mail( $admin_email, $subject, $body );
}

?>